`include "timescale.v"

module aes_fsm(CLK, nRST, start, done, busy, ld);
   input  CLK;
   input  nRST;
   input  start;
   input  done;
   output busy;
   output ld;

   reg 	       busy;
   reg 	       ld;
   reg [1:0]   state;
   reg [1:0]   next_state;

   parameter [1:0] STATE_IDLE = 2'b00,
		   STATE_LOAD = 2'b01,
		   STATE_BUSY = 2'b10,
		   STATE_DONE = 2'b11;

   // combinational logic for state machine
   always @(state or start or done)
     case (state)
       STATE_IDLE:
	 if (start) next_state <= STATE_LOAD;
	 else       next_state <= STATE_IDLE;
       STATE_LOAD:
	            next_state <= STATE_BUSY;
       STATE_BUSY:
	 if (done)  next_state <= STATE_DONE;
         else       next_state <= STATE_BUSY;
       STATE_DONE:
	 if (start) next_state <= STATE_LOAD;
	 else       next_state <= STATE_DONE;
       default:
	            next_state <= STATE_IDLE;
     endcase

   // FF for state machine
   always @(posedge CLK or negedge nRST)
     if (!nRST)
       state <= STATE_IDLE;
     else
       state <= next_state;

   // output FF
   always @(posedge CLK or negedge nRST)
      if (!nRST) begin
	 ld <= 1'b0;
	 busy <= 1'b0;
      end
      else if (state == STATE_IDLE) begin
	 ld <= 1'b0;
	 busy <= 1'b0;
      end
      else if (state == STATE_LOAD) begin
	 ld <= 1'b1;
	 busy <= 1'b1;
      end
      else if (state == STATE_BUSY) begin
	 ld <= 1'b0;
	 busy <= 1'b1;
      end
      else if (state == STATE_DONE) begin
	 ld <= 1'b0;
	 busy <= 1'b0;
      end

endmodule
