//
// AES128-CBC Encryption/Decryption for H8/300H
//

`include "timescale.v"

module aes_h8(CLK, nRST, nCS, nWE, nOE, nAS, ADDR, DATA);
   input       CLK;
   input       nRST;
   input       nCS;
   input       nWE;
   input       nOE;
   input       nAS;
   input [6:0] ADDR;
   inout [7:0] DATA;

   wire [127:0] key;
   wire [127:0] din;
   wire [127:0] iv;
   wire [127:0] dout;
   wire 	start;
   wire 	decrypt;
   wire 	cbc;
   wire 	done;

   // H8 bus I/F
   aes_h8_busif aes_h8_busif(.CLK(CLK), .nRST(nRST),
			     .nCS(nCS), .nWE(nWE), .nOE(nOE), .nAS(nAS),
			     .ADDR(ADDR), .DATA(DATA),
			     .key(key), .din(din), .iv(iv), .dout(dout),
			     .busy(busy), .done(done), .start(start),
			     .decrypt(decrypt), .cbc(cbc));

   // FSM for AES Core
   wire 	ld;
   aes_fsm aes_fsm(.CLK(CLK), .nRST(nRST),
		   .start(start), .done(done),
		   .busy(busy), .ld(ld));

   // AES Core (with CBC mode)
   wire [127:0] dout_cbc;
   wire [127:0] din_cbc;
   assign 	dout = decrypt ? (cbc ? dout_cbc ^ iv : dout_cbc) : dout_cbc;
   assign 	din_cbc = decrypt ? din : (cbc ? din ^ iv : din);
   aes aes(.clk(CLK), .reset(nRST),
	   .load_i(ld), .decrypt_i(decrypt),
	   .data_i(din_cbc), .key_i(key),
	   .ready_o(done),
	   .data_o(dout_cbc));

endmodule
