`include "timescale.v"
`include "aes_regdef.v"

module aes_h8_busif(CLK, nRST,
		    nCS, nWE, nOE, nAS, ADDR, DATA,
		    key, din, iv, dout,
		    done, busy, start, decrypt, cbc);
   input       CLK;
   input       nRST;
   input       nCS;
   input       nWE;
   input       nOE;
   input       nAS;
   input [6:0] ADDR;
   inout [7:0] DATA;
   output [127:0] key;
   output [127:0] din;
   output [127:0] iv;
   input [127:0]  dout;
   input 	  done;
   input 	  busy;
   output 	  start;
   output 	  decrypt;
   output 	  cbc;

   reg 		  cbc;
   reg 		  decrypt;

   wire [7:0] 	  KEY_0, KEY_1, KEY_2, KEY_3,
		  KEY_4, KEY_5, KEY_6, KEY_7,
		  KEY_8, KEY_9, KEY_10, KEY_11,
	       KEY_12, KEY_13, KEY_14, KEY_15;
   wire [7:0]  DIN_0, DIN_1, DIN_2, DIN_3,
	       DIN_4, DIN_5, DIN_6, DIN_7,
	       DIN_8, DIN_9, DIN_10, DIN_11,
	       DIN_12, DIN_13, DIN_14, DIN_15;
   wire [7:0]  IV_0, IV_1, IV_2, IV_3,
	       IV_4, IV_5, IV_6, IV_7,
               IV_8, IV_9, IV_10, IV_11,
	       IV_12, IV_13, IV_14, IV_15;
   wire [7:0]  DOUT_0, DOUT_1, DOUT_2, DOUT_3,
	       DOUT_4, DOUT_5, DOUT_6, DOUT_7,
               DOUT_8, DOUT_9, DOUT_10, DOUT_11,
	       DOUT_12, DOUT_13, DOUT_14, DOUT_15;
   wire [7:0]  CTRL;
`ifdef XXX
   wire [7:0]  STATUS;
`else
   wire        STATUS;
`endif

   assign 	key = {KEY_0, KEY_1, KEY_2, KEY_3,
		       KEY_4, KEY_5, KEY_6, KEY_7,
   		       KEY_8, KEY_9, KEY_10, KEY_11,
		       KEY_12, KEY_13, KEY_14, KEY_15};
   assign 	din = {DIN_0, DIN_1, DIN_2, DIN_3,
		       DIN_4, DIN_5, DIN_6, DIN_7,
		       DIN_8, DIN_9, DIN_10, DIN_11,
		       DIN_12, DIN_13, DIN_14, DIN_15};
   assign 	iv = {IV_0, IV_1, IV_2, IV_3,
		      IV_4, IV_5, IV_6, IV_7,
		      IV_8, IV_9, IV_10, IV_11,
		      IV_12, IV_13, IV_14, IV_15};

   // Chip Select Signals
   wire 	CS_KEY_0, CS_KEY_1, CS_KEY_2, CS_KEY_3,
		CS_KEY_4, CS_KEY_5, CS_KEY_6, CS_KEY_7,
		CS_KEY_8, CS_KEY_9, CS_KEY_10, CS_KEY_11,
		CS_KEY_12, CS_KEY_13, CS_KEY_14, CS_KEY_15;
   wire 	CS_DIN_0, CS_DIN_1, CS_DIN_2, CS_DIN_3,
		CS_DIN_4, CS_DIN_5, CS_DIN_6, CS_DIN_7,
		CS_DIN_8, CS_DIN_9, CS_DIN_10, CS_DIN_11,
		CS_DIN_12, CS_DIN_13, CS_DIN_14, CS_DIN_15;
   wire 	CS_IV_0, CS_IV_1, CS_IV_2, CS_IV_3,
		CS_IV_4, CS_IV_5, CS_IV_6, CS_IV_7,
		CS_IV_8, CS_IV_9, CS_IV_10, CS_IV_11,
		CS_IV_12, CS_IV_13, CS_IV_14, CS_IV_15;
   wire 	CS_DOUT_0, CS_DOUT_1, CS_DOUT_2, CS_DOUT_3,
		CS_DOUT_4, CS_DOUT_5, CS_DOUT_6, CS_DOUT_7,
		CS_DOUT_8, CS_DOUT_9, CS_DOUT_10, CS_DOUT_11,
		CS_DOUT_12, CS_DOUT_13, CS_DOUT_14, CS_DOUT_15;
   wire 	CS_CTRL;
   wire 	CS_STATUS;
   wire 	CS_UNASSIGN;

   // Address Decoder
   assign 	CS_KEY_0   = ~nCS & ~nAS & (ADDR == `AES_KEY_0);
   assign 	CS_KEY_1   = ~nCS & ~nAS & (ADDR == `AES_KEY_1);
   assign 	CS_KEY_2   = ~nCS & ~nAS & (ADDR == `AES_KEY_2);
   assign 	CS_KEY_3   = ~nCS & ~nAS & (ADDR == `AES_KEY_3);
   assign 	CS_KEY_4   = ~nCS & ~nAS & (ADDR == `AES_KEY_4);
   assign 	CS_KEY_5   = ~nCS & ~nAS & (ADDR == `AES_KEY_5);
   assign 	CS_KEY_6   = ~nCS & ~nAS & (ADDR == `AES_KEY_6);
   assign 	CS_KEY_7   = ~nCS & ~nAS & (ADDR == `AES_KEY_7);
   assign 	CS_KEY_8   = ~nCS & ~nAS & (ADDR == `AES_KEY_8);
   assign 	CS_KEY_9   = ~nCS & ~nAS & (ADDR == `AES_KEY_9);
   assign 	CS_KEY_10  = ~nCS & ~nAS & (ADDR == `AES_KEY_10);
   assign 	CS_KEY_11  = ~nCS & ~nAS & (ADDR == `AES_KEY_11);
   assign 	CS_KEY_12  = ~nCS & ~nAS & (ADDR == `AES_KEY_12);
   assign 	CS_KEY_13  = ~nCS & ~nAS & (ADDR == `AES_KEY_13);
   assign 	CS_KEY_14  = ~nCS & ~nAS & (ADDR == `AES_KEY_14);
   assign 	CS_KEY_15  = ~nCS & ~nAS & (ADDR == `AES_KEY_15);
   assign 	CS_DIN_0   = ~nCS & ~nAS & (ADDR == `AES_DIN_0);
   assign 	CS_DIN_1   = ~nCS & ~nAS & (ADDR == `AES_DIN_1);
   assign 	CS_DIN_2   = ~nCS & ~nAS & (ADDR == `AES_DIN_2);
   assign 	CS_DIN_3   = ~nCS & ~nAS & (ADDR == `AES_DIN_3);
   assign 	CS_DIN_4   = ~nCS & ~nAS & (ADDR == `AES_DIN_4);
   assign 	CS_DIN_5   = ~nCS & ~nAS & (ADDR == `AES_DIN_5);
   assign 	CS_DIN_6   = ~nCS & ~nAS & (ADDR == `AES_DIN_6);
   assign 	CS_DIN_7   = ~nCS & ~nAS & (ADDR == `AES_DIN_7);
   assign 	CS_DIN_8   = ~nCS & ~nAS & (ADDR == `AES_DIN_8);
   assign 	CS_DIN_9   = ~nCS & ~nAS & (ADDR == `AES_DIN_9);
   assign 	CS_DIN_10  = ~nCS & ~nAS & (ADDR == `AES_DIN_10);
   assign 	CS_DIN_11  = ~nCS & ~nAS & (ADDR == `AES_DIN_11);
   assign 	CS_DIN_12  = ~nCS & ~nAS & (ADDR == `AES_DIN_12);
   assign 	CS_DIN_13  = ~nCS & ~nAS & (ADDR == `AES_DIN_13);
   assign 	CS_DIN_14  = ~nCS & ~nAS & (ADDR == `AES_DIN_14);
   assign 	CS_DIN_15  = ~nCS & ~nAS & (ADDR == `AES_DIN_15);
   assign 	CS_IV_0    = ~nCS & ~nAS & (ADDR == `AES_IV_0);
   assign 	CS_IV_1    = ~nCS & ~nAS & (ADDR == `AES_IV_1);
   assign 	CS_IV_2    = ~nCS & ~nAS & (ADDR == `AES_IV_2);
   assign 	CS_IV_3    = ~nCS & ~nAS & (ADDR == `AES_IV_3);
   assign 	CS_IV_4    = ~nCS & ~nAS & (ADDR == `AES_IV_4);
   assign 	CS_IV_5    = ~nCS & ~nAS & (ADDR == `AES_IV_5);
   assign 	CS_IV_6    = ~nCS & ~nAS & (ADDR == `AES_IV_6);
   assign 	CS_IV_7    = ~nCS & ~nAS & (ADDR == `AES_IV_7);
   assign 	CS_IV_8    = ~nCS & ~nAS & (ADDR == `AES_IV_8);
   assign 	CS_IV_9    = ~nCS & ~nAS & (ADDR == `AES_IV_9);
   assign 	CS_IV_10   = ~nCS & ~nAS & (ADDR == `AES_IV_10);
   assign 	CS_IV_11   = ~nCS & ~nAS & (ADDR == `AES_IV_11);
   assign 	CS_IV_12   = ~nCS & ~nAS & (ADDR == `AES_IV_12);
   assign 	CS_IV_13   = ~nCS & ~nAS & (ADDR == `AES_IV_13);
   assign 	CS_IV_14   = ~nCS & ~nAS & (ADDR == `AES_IV_14);
   assign 	CS_IV_15   = ~nCS & ~nAS & (ADDR == `AES_IV_15);
   assign 	CS_DOUT_0  = ~nCS & ~nAS & (ADDR == `AES_DOUT_0);
   assign 	CS_DOUT_1  = ~nCS & ~nAS & (ADDR == `AES_DOUT_1);
   assign 	CS_DOUT_2  = ~nCS & ~nAS & (ADDR == `AES_DOUT_2);
   assign 	CS_DOUT_3  = ~nCS & ~nAS & (ADDR == `AES_DOUT_3);
   assign 	CS_DOUT_4  = ~nCS & ~nAS & (ADDR == `AES_DOUT_4);
   assign 	CS_DOUT_5  = ~nCS & ~nAS & (ADDR == `AES_DOUT_5);
   assign 	CS_DOUT_6  = ~nCS & ~nAS & (ADDR == `AES_DOUT_6);
   assign 	CS_DOUT_7  = ~nCS & ~nAS & (ADDR == `AES_DOUT_7);
   assign 	CS_DOUT_8  = ~nCS & ~nAS & (ADDR == `AES_DOUT_8);
   assign 	CS_DOUT_9  = ~nCS & ~nAS & (ADDR == `AES_DOUT_9);
   assign 	CS_DOUT_10 = ~nCS & ~nAS & (ADDR == `AES_DOUT_10);
   assign 	CS_DOUT_11 = ~nCS & ~nAS & (ADDR == `AES_DOUT_11);
   assign 	CS_DOUT_12 = ~nCS & ~nAS & (ADDR == `AES_DOUT_12);
   assign 	CS_DOUT_13 = ~nCS & ~nAS & (ADDR == `AES_DOUT_13);
   assign 	CS_DOUT_14 = ~nCS & ~nAS & (ADDR == `AES_DOUT_14);
   assign 	CS_DOUT_15 = ~nCS & ~nAS & (ADDR == `AES_DOUT_15);
   assign 	CS_CTRL    = ~nCS & ~nAS & (ADDR == `AES_CTRL);
   assign 	CS_STATUS  = ~nCS & ~nAS & (ADDR == `AES_STATUS);
   assign	CS_UNASSIGN = ~nCS & ~nAS & (ADDR > `AES_STATUS);

   // Bus I/F Registers
   aes_ifreg #(8) REG_KEY_0(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_0 & ~nWE),
			    .R(KEY_0), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_1(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_1 & ~nWE),
			    .R(KEY_1), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_2(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_2 & ~nWE),
			    .R(KEY_2), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_3(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_3 & ~nWE),
			    .R(KEY_3), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_4(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_4 & ~nWE),
			    .R(KEY_4), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_5(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_5 & ~nWE),
			    .R(KEY_5), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_6(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_6 & ~nWE),
			    .R(KEY_6), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_7(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_7 & ~nWE),
			    .R(KEY_7), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_8(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_8 & ~nWE),
			    .R(KEY_8), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_9(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_9 & ~nWE),
			    .R(KEY_9), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_10(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_10 & ~nWE),
			     .R(KEY_10), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_11(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_11 & ~nWE),
			     .R(KEY_11), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_12(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_12 & ~nWE),
			     .R(KEY_12), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_13(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_13 & ~nWE),
			     .R(KEY_13), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_14(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_14 & ~nWE),
			     .R(KEY_14), .DIN(DATA));
   aes_ifreg #(8) REG_KEY_15(.CLK(CLK), .nRST(nRST), .EN(CS_KEY_15 & ~nWE),
			     .R(KEY_15), .DIN(DATA));

   aes_ifreg #(8) REG_DIN_0(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_0 & ~nWE),
			    .R(DIN_0), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_1(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_1 & ~nWE),
			    .R(DIN_1), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_2(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_2 & ~nWE),
			    .R(DIN_2), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_3(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_3 & ~nWE),
			    .R(DIN_3), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_4(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_4 & ~nWE),
			    .R(DIN_4), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_5(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_5 & ~nWE),
			    .R(DIN_5), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_6(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_6 & ~nWE),
			    .R(DIN_6), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_7(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_7 & ~nWE),
			    .R(DIN_7), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_8(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_8 & ~nWE),
			    .R(DIN_8), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_9(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_9 & ~nWE),
			    .R(DIN_9), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_10(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_10 & ~nWE),
			     .R(DIN_10), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_11(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_11 & ~nWE),
			     .R(DIN_11), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_12(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_12 & ~nWE),
			     .R(DIN_12), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_13(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_13 & ~nWE),
			     .R(DIN_13), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_14(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_14 & ~nWE),
			     .R(DIN_14), .DIN(DATA));
   aes_ifreg #(8) REG_DIN_15(.CLK(CLK), .nRST(nRST), .EN(CS_DIN_15 & ~nWE),
			     .R(DIN_15), .DIN(DATA));

   aes_ifreg #(8) REG_IV_0(.CLK(CLK), .nRST(nRST), .EN(CS_IV_0 & ~nWE),
			   .R(IV_0), .DIN(DATA));
   aes_ifreg #(8) REG_IV_1(.CLK(CLK), .nRST(nRST), .EN(CS_IV_1 & ~nWE),
			   .R(IV_1), .DIN(DATA));
   aes_ifreg #(8) REG_IV_2(.CLK(CLK), .nRST(nRST), .EN(CS_IV_2 & ~nWE),
			   .R(IV_2), .DIN(DATA));
   aes_ifreg #(8) REG_IV_3(.CLK(CLK), .nRST(nRST), .EN(CS_IV_3 & ~nWE),
			   .R(IV_3), .DIN(DATA));
   aes_ifreg #(8) REG_IV_4(.CLK(CLK), .nRST(nRST), .EN(CS_IV_4 & ~nWE),
			   .R(IV_4), .DIN(DATA));
   aes_ifreg #(8) REG_IV_5(.CLK(CLK), .nRST(nRST), .EN(CS_IV_5 & ~nWE),
			   .R(IV_5), .DIN(DATA));
   aes_ifreg #(8) REG_IV_6(.CLK(CLK), .nRST(nRST), .EN(CS_IV_6 & ~nWE),
			   .R(IV_6), .DIN(DATA));
   aes_ifreg #(8) REG_IV_7(.CLK(CLK), .nRST(nRST), .EN(CS_IV_7 & ~nWE),
			   .R(IV_7), .DIN(DATA));
   aes_ifreg #(8) REG_IV_8(.CLK(CLK), .nRST(nRST), .EN(CS_IV_8 & ~nWE),
			   .R(IV_8), .DIN(DATA));
   aes_ifreg #(8) REG_IV_9(.CLK(CLK), .nRST(nRST), .EN(CS_IV_9 & ~nWE),
			   .R(IV_9), .DIN(DATA));
   aes_ifreg #(8) REG_IV_10(.CLK(CLK), .nRST(nRST), .EN(CS_IV_10 & ~nWE),
			    .R(IV_10), .DIN(DATA));
   aes_ifreg #(8) REG_IV_11(.CLK(CLK), .nRST(nRST), .EN(CS_IV_11 & ~nWE),
			    .R(IV_11), .DIN(DATA));
   aes_ifreg #(8) REG_IV_12(.CLK(CLK), .nRST(nRST), .EN(CS_IV_12 & ~nWE),
			    .R(IV_12), .DIN(DATA));
   aes_ifreg #(8) REG_IV_13(.CLK(CLK), .nRST(nRST), .EN(CS_IV_13 & ~nWE),
			    .R(IV_13), .DIN(DATA));
   aes_ifreg #(8) REG_IV_14(.CLK(CLK), .nRST(nRST), .EN(CS_IV_14 & ~nWE),
			    .R(IV_14), .DIN(DATA));
   aes_ifreg #(8) REG_IV_15(.CLK(CLK), .nRST(nRST), .EN(CS_IV_15 & ~nWE),
			    .R(IV_15), .DIN(DATA));

   aes_ifreg #(8) REG_DOUT_0(.CLK(CLK), .nRST(nRST), .EN(done),
			     .R(DOUT_0), .DIN(dout[127:120]));
   aes_ifreg #(8) REG_DOUT_1(.CLK(CLK), .nRST(nRST), .EN(done),
			     .R(DOUT_1), .DIN(dout[119:112]));
   aes_ifreg #(8) REG_DOUT_2(.CLK(CLK), .nRST(nRST), .EN(done),
			     .R(DOUT_2), .DIN(dout[111:104]));
   aes_ifreg #(8) REG_DOUT_3(.CLK(CLK), .nRST(nRST), .EN(done),
			     .R(DOUT_3), .DIN(dout[103:96]));
   aes_ifreg #(8) REG_DOUT_4(.CLK(CLK), .nRST(nRST), .EN(done),
			     .R(DOUT_4), .DIN(dout[95:88]));
   aes_ifreg #(8) REG_DOUT_5(.CLK(CLK), .nRST(nRST), .EN(done),
			     .R(DOUT_5), .DIN(dout[87:80]));
   aes_ifreg #(8) REG_DOUT_6(.CLK(CLK), .nRST(nRST), .EN(done),
			     .R(DOUT_6), .DIN(dout[79:72]));
   aes_ifreg #(8) REG_DOUT_7(.CLK(CLK), .nRST(nRST), .EN(done),
			     .R(DOUT_7), .DIN(dout[71:64]));
   aes_ifreg #(8) REG_DOUT_8(.CLK(CLK), .nRST(nRST), .EN(done),
			     .R(DOUT_8), .DIN(dout[63:56]));
   aes_ifreg #(8) REG_DOUT_9(.CLK(CLK), .nRST(nRST), .EN(done),
			     .R(DOUT_9), .DIN(dout[55:48]));
   aes_ifreg #(8) REG_DOUT_10(.CLK(CLK), .nRST(nRST), .EN(done),
			      .R(DOUT_10), .DIN(dout[47:40]));
   aes_ifreg #(8) REG_DOUT_11(.CLK(CLK), .nRST(nRST), .EN(done),
			      .R(DOUT_11), .DIN(dout[39:32]));
   aes_ifreg #(8) REG_DOUT_12(.CLK(CLK), .nRST(nRST), .EN(done),
			      .R(DOUT_12), .DIN(dout[31:24]));
   aes_ifreg #(8) REG_DOUT_13(.CLK(CLK), .nRST(nRST), .EN(done),
			      .R(DOUT_13), .DIN(dout[23:16]));
   aes_ifreg #(8) REG_DOUT_14(.CLK(CLK), .nRST(nRST), .EN(done),
			      .R(DOUT_14), .DIN(dout[15:8]));
   aes_ifreg #(8) REG_DOUT_15(.CLK(CLK), .nRST(nRST), .EN(done),
			      .R(DOUT_15), .DIN(dout[7:0]));

   // Control Register (with start bit clear)
   wire [7:0] 	c = start ? CTRL & ~`AES_CTRL_START : DATA;
   aes_ifreg #(8) REG_CTRL(.CLK(CLK), .nRST(nRST),
			   .EN((CS_CTRL & ~nWE) | start),
			   .R(CTRL), .DIN(c));

   // Generate start trigger pulse
   reg 		start_ff_0;
   reg 		start_ff_1;
   always @(posedge CLK)
     if ((CTRL & `AES_CTRL_START) == `AES_CTRL_START)
       start_ff_0 <= 1'b1;
     else
       start_ff_0 <= 1'b0;
   always @(posedge CLK)
     start_ff_1 <= start_ff_0;
   assign 	start = start_ff_0 & ~start_ff_1;
   
   // decrypt?
   always @(posedge CLK or negedge nRST)
     if (!nRST)
       decrypt <= 1'b0;
     else if ((CTRL & `AES_CTRL_DEC) == `AES_CTRL_DEC)
       decrypt <= 1'b1;
     else
       decrypt <= 1'b0;

   // CBC mode
   always @(posedge CLK or negedge nRST)
     if (!nRST)
       cbc <= 1'b0;
     else if ((CTRL & `AES_CTRL_CBC) == `AES_CTRL_CBC)
       cbc <= 1'b1;
     else
       cbc <= 1'b0;
   
   // Status Register
`ifdef XXX
   aes_ifreg #(8) REG_STATUS(.CLK(CLK), .nRST(nRST), .EN(1'b1),
			     .R(STATUS), .DIN({busy, 7'b0000000}));
`else
   aes_ifreg #(1) REG_STATUS(.CLK(CLK), .nRST(nRST), .EN(1'b1),
			     .R(STATUS), .DIN(busy));
`endif

   // Data Bus Output
   assign 	DATA =
		(CS_KEY_0 & ~nOE) ? KEY_0 :
		(CS_KEY_1 & ~nOE) ? KEY_1 :
		(CS_KEY_2 & ~nOE) ? KEY_2 :
		(CS_KEY_3 & ~nOE) ? KEY_3 :
		(CS_KEY_4 & ~nOE) ? KEY_4 :
		(CS_KEY_5 & ~nOE) ? KEY_5 :
		(CS_KEY_6 & ~nOE) ? KEY_6 :
		(CS_KEY_7 & ~nOE) ? KEY_7 :
		(CS_KEY_8 & ~nOE) ? KEY_8 :
		(CS_KEY_9 & ~nOE) ? KEY_9 :
		(CS_KEY_10 & ~nOE) ? KEY_10 :
		(CS_KEY_11 & ~nOE) ? KEY_11 :
		(CS_KEY_12 & ~nOE) ? KEY_12 :
		(CS_KEY_13 & ~nOE) ? KEY_13 :
		(CS_KEY_14 & ~nOE) ? KEY_14 :
		(CS_KEY_15 & ~nOE) ? KEY_15 :
		(CS_DIN_0 & ~nOE) ? DIN_0 :
		(CS_DIN_1 & ~nOE) ? DIN_1 :
		(CS_DIN_2 & ~nOE) ? DIN_2 :
		(CS_DIN_3 & ~nOE) ? DIN_3 :
		(CS_DIN_4 & ~nOE) ? DIN_4 :
		(CS_DIN_5 & ~nOE) ? DIN_5 :
		(CS_DIN_6 & ~nOE) ? DIN_6 :
		(CS_DIN_7 & ~nOE) ? DIN_7 :
		(CS_DIN_8 & ~nOE) ? DIN_8 :
		(CS_DIN_9 & ~nOE) ? DIN_9 :
		(CS_DIN_10 & ~nOE) ? DIN_10 :
		(CS_DIN_11 & ~nOE) ? DIN_11 :
		(CS_DIN_12 & ~nOE) ? DIN_12 :
		(CS_DIN_13 & ~nOE) ? DIN_13 :
		(CS_DIN_14 & ~nOE) ? DIN_14 :
		(CS_DIN_15 & ~nOE) ? DIN_15 :
		(CS_IV_0 & ~nOE) ? IV_0 :
		(CS_IV_1 & ~nOE) ? IV_1 :
		(CS_IV_2 & ~nOE) ? IV_2 :
		(CS_IV_3 & ~nOE) ? IV_3 :
		(CS_IV_4 & ~nOE) ? IV_4 :
		(CS_IV_5 & ~nOE) ? IV_5 :
		(CS_IV_6 & ~nOE) ? IV_6 :
		(CS_IV_7 & ~nOE) ? IV_7 :
		(CS_IV_8 & ~nOE) ? IV_8 :
		(CS_IV_9 & ~nOE) ? IV_9 :
		(CS_IV_10 & ~nOE) ? IV_10 :
		(CS_IV_11 & ~nOE) ? IV_11 :
		(CS_IV_12 & ~nOE) ? IV_12 :
		(CS_IV_13 & ~nOE) ? IV_13 :
		(CS_IV_14 & ~nOE) ? IV_14 :
		(CS_IV_15 & ~nOE) ? IV_15 :
		(CS_DOUT_0 & ~nOE) ? DOUT_0 :
		(CS_DOUT_1 & ~nOE) ? DOUT_1 :
		(CS_DOUT_2 & ~nOE) ? DOUT_2 :
		(CS_DOUT_3 & ~nOE) ? DOUT_3 :
		(CS_DOUT_4 & ~nOE) ? DOUT_4 :
		(CS_DOUT_5 & ~nOE) ? DOUT_5 :
		(CS_DOUT_6 & ~nOE) ? DOUT_6 :
		(CS_DOUT_7 & ~nOE) ? DOUT_7 :
		(CS_DOUT_8 & ~nOE) ? DOUT_8 :
		(CS_DOUT_9 & ~nOE) ? DOUT_9 :
		(CS_DOUT_10 & ~nOE) ? DOUT_10 :
		(CS_DOUT_11 & ~nOE) ? DOUT_11 :
		(CS_DOUT_12 & ~nOE) ? DOUT_12 :
		(CS_DOUT_13 & ~nOE) ? DOUT_13 :
		(CS_DOUT_14 & ~nOE) ? DOUT_14 :
		(CS_DOUT_15 & ~nOE) ? DOUT_15 :
		(CS_CTRL & ~nOE) ? CTRL :
`ifdef XXX
		(CS_STATUS & ~nOE) ? STATUS :
`else
		(CS_STATUS & ~nOE) ? {STATUS, 7'b0} :
`endif
		(CS_UNASSIGN & ~nOE) ? 8'b0 :
		8'bz;

endmodule
